<script type="text/x-template">
  <article>
    <h2>{{title}}</h2>
    <p>{{description}}</p>
  </article>
</script>
<script type="text/x-template">
  article
    h2 {{title}}
    p {{description}}
</script>