<script id="user-template" type="text/template">
  <div id="user">
    <h1><%= user.name %></h1>
    <p><%= user.description %></p>
  </div>
</script>
<script id="user-template" type="text/template">
  if (foo) {
    bar();
  }
</script>