<foo attr="&lt;%= bar %&gt;"></foo>
<foo class="&lt;%= bar %&gt;"></foo>
<foo attr="<%= bar %>"></foo>
<foo class="<%= bar %>"></foo>
<foo class="<%= bar %> lol rofl"></foo>
<foo class="<%= bar %> lol rofl <%= lmao %>"></foo>