
@footer-height: 40px;
@footer-height-narrow: 60px;
@footer-padding: 10px;

/* Sticky footer styles */
html, body {
  height: 100%;
  padding: 0;
  margin: 0;
}

// Wrapper for page content to push down footer
#wrap {
  min-height: 100%; height: auto !important; height: 100%;
  // Negative indent footer by it's height
  margin: 0 auto -@footer-height;
}

// Set the fixed height of the footer here
#push, #footer-wrapper { height: @footer-height; }
#footer-wrapper { background-color: #f5f5f5; }

footer { padding: @footer-padding 0; border-top: 1px solid #e5e5e5; background-color: #f5f5f5; }

footer p { margin-bottom: 0; color: #777; }

// Lastly, apply responsive CSS fixes as necessary
@media (max-width: 767px) {
  #push, #footer-wrapper { height: @footer-height-narrow; }
  #wrap { margin: 0 auto -@footer-height-narrow; }
}