dl.parameter-list, dl.returns {
    margin-left: 1em;
    margin-bottom: 0;
}

dl.returns {
    margin-top: 1em;
}
dl.parameter-list, dl.returns {
  dt {
      color: #666;
      float: left;
      margin-right: 16px;
      min-width: 110px;
  }
  dd.type {
      margin: 0;
      float: left;
      -moz-border-radius: 5px;
      -webkit-border-radius: 5px;
      border-radius: 5px;
      background: #CCC;
      font-size: .75em;
      min-width: 80px;
      margin: 0 8px 0 0;
      padding: 2px 5px;
      text-align: center;
  }
  dd.description {
      display: table;
      min-height: 24px;
  }

  dd.type.string {
      background: #e1edb1;
      color: #3d4c00;
  }
  dd.type.object {
      background: #edb1b1;
      color: #4c0000;
  }
  dd.type.function {
      background: #cfb1ed;
      color: #26004c;
  }
  dd.type.number {
      background: #b1c9ed;
      color: #001e4c;
  }
  dd.type.boolean {
      background: #b1edc9;
      color: #004c1e;
  }
  dd.type.array {
      background: #edd5b1;
      color: #4c2d00;
  }
}
